package ueb041midiplayer;

import processing.core.PApplet;

import java.io.*;

import javax.sound.midi.*;

import java.util.*;
import java.util.concurrent.TimeUnit;

import ddf.minim.*;
import ddf.minim.analysis.*;

@SuppressWarnings("serial")
public class Ueb041MidiPlayer extends PApplet {

	private boolean isStoped;
	private int currentSongIndex = 0;
	private Sequencer sequencer;
	private List<Song> songs = new ArrayList<Song>();
	private FFT fft;
	private AudioInput audioInput;
	private Minim minim;

	public void setup() {

		// Grsse setzen
		size(500, 200);

		smooth();

		// Den Sequenzer initialisieren
		try {
			sequencer = MidiSystem.getSequencer();
			sequencer.open();
		} catch (Exception ex) {
			ex.printStackTrace();

			exit();
			return;
		}

		// Alle Songs auslesen im Verzeichnis Sounds
		if (!getSongs("songs")) {
			System.out.println("Keine Songs verfgbar");

			exit();
			return;
		}

		// Minim und FFT initialisieren
		minim = new Minim(this);
		// BUG:
		// - Nimmt den Mikrofon-Eingang
		audioInput = minim.getLineIn(Minim.MONO, 16384, 44100);

		fft = new FFT(audioInput.bufferSize(), audioInput.sampleRate());
		fft.window(FFT.HAMMING);

		playSong();
	}

	public void draw() {

		// Zeichne Hintergrund
		background(0);

		// Zeichne Wellenform
		stroke(255);
		saturation(10);

		for (int index = 0; index < audioInput.bufferSize() - 1; index++) {
			line(index, 80 + audioInput.mix.get(index) * 50, index + 1,
					80 + audioInput.mix.get(index + 1) * 50);
		}

		// Draw FFT
		stroke(175);
		fft.forward(audioInput.mix);
		for (int index = 0; index < fft.specSize(); index++) {
			line(index, height - 20, index, height - 20 - fft.getBand(index)
					* 4);
		}

		fill(50);
		rect(0, height - 20, width, 20);

		stroke(100);
		fill(100);
		rect(0,
				height - 20,
				(int) (width / (float) sequencer.getMicrosecondLength() * sequencer
						.getMicrosecondPosition()), 20);

		fill(200);

		// Zentriert den Text
		textAlign(CENTER, CENTER);

		// Zeichnet den Text
		text(songs.get(currentSongIndex).getName(), 2, 0, width, height - 20);
		text(String.format(
				"%d:%02d",
				TimeUnit.MICROSECONDS.toMinutes(sequencer
						.getMicrosecondPosition()),
				TimeUnit.MICROSECONDS.toSeconds(sequencer
						.getMicrosecondPosition())
						- TimeUnit.MINUTES.toSeconds(TimeUnit.MICROSECONDS
								.toMinutes(sequencer.getMicrosecondPosition())))
				+ " / "
				+ String.format(
						"%d:%02d",
						TimeUnit.MICROSECONDS.toMinutes(sequencer
								.getMicrosecondLength()),
						TimeUnit.MICROSECONDS.toSeconds(sequencer
								.getMicrosecondLength())
								- TimeUnit.MINUTES.toSeconds(TimeUnit.MICROSECONDS
										.toMinutes(sequencer
												.getMicrosecondLength()))), 2,
				height - 20, width, 20);

		// Prfen, ob zum nchsten Sound weiter gegangen werden soll
		shouldPlayNext();
	}

	public void keyPressed() {
		switch (key) {
		case ' ':
			if (sequencer.isRunning()) {
				pauseSong();
			} else {
				playSong();
			}

			break;
		case 'n':
			next();
			break;
		}
	}

	public void mousePressed() {

		if (mouseY < height - 20)
			return;

		int pos = (int) (((float) mouseX / width) * sequencer.getTickLength());

		sequencer.setTickPosition(pos);
	}

	/**
	 * Prft, ob der nchste Sound gespielt werden soll
	 */
	private void shouldPlayNext() {
		if (!isStoped && !sequencer.isRunning()) {
			next();
		}
	}

	/**
	 * Wechselt zum nchsten Lied
	 */
	private void next() {

		sequencer.stop();
		sequencer.setTickPosition(0);

		if (++currentSongIndex >= songs.size()) {
			currentSongIndex = 0;
		}

		playSong();

		System.out.println("Next..");
	}

	/**
	 * Startet den Player
	 */
	private void playSong() {

		if (sequencer.isRunning())
			return;

		try {
			long currentPosition = sequencer.getTickPosition();
			sequencer.setSequence(songs.get(currentSongIndex).getSequence());
			sequencer.setTickPosition(currentPosition);
			sequencer.start();
		} catch (Exception ex) {
			ex.printStackTrace();
		}

		isStoped = false;
	}

	/**
	 * Pausiert den Player
	 */
	private void pauseSong() {
		isStoped = true;

		if (!sequencer.isRunning())
			return;

		sequencer.stop();
	}

	/**
	 * Liest alle Sounds in dem bergebenen Verzeichnis aus
	 * 
	 * @param directory
	 * @return Gibt true zurck, falls Sounds gefunden wurden; andernfalls false
	 */
	private boolean getSongs(String directory) {

		File dataDirectory = new File(directory);
		if (dataDirectory == null || !dataDirectory.isDirectory())
			return false;

		for (File file : dataDirectory.listFiles()) {

			// Filter .mid files
			if (!file.getName().endsWith(".mid"))
				continue;

			try {
				songs.add(new Song(file, MidiSystem.getSequence(file)));
				System.out.println(songs.get(songs.size() - 1).getName()
						+ " added to the song list");
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}

		return songs.size() > 0;
	}
}